﻿-- TODO: Doing list commands in the channel you are in!
-- Macro Profiles
local MPS_Version = "2.4.1.60100"
local WoWversion = 60100
MPSDEV = 0 -- Developer tag, turn this on and you will be filled with logs!
MPSMACROIDERROR = nil --- If anything is wrong with a Macro, enable debuging by filling in MacroID
MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")
local AceGUI = LibStub("AceGUI-3.0")

--Char specific macros
minMacros = 1
MPS.timerCount = 0

function MPS:OnInitialize()
	MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Default")
	MPglobaldb = LibStub("AceDB-3.0"):New("ProfilesMacrosG",nil,"Default")
	MPS:SetupOptions()
   
   -- Correct WoW version!
	if select(4, GetBuildInfo()) == WoWversion then
	
		MPS:UnregisterEvent("PVP_REWARDS_UPDATE")
	
		-- Save a restore point if empty db
		if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
			MPSSAVERESTORE=1 
			MPS:SaveMacroProfile("Restore [LOCKED]") 
			MPSSAVERESTORE=nil
		end
	
		if MPdb.char.Active_profile and MPdb.char.MPSLoadProfileAtStart == 1 then
			if MPdb.char.Active_profile ~= "Restore [LOCKED]" then
				MPS:LoadMacroProfile(MPdb.char.Active_profile)
			end
		end
	end
		
	-- Different WoW version!
	if select(4, GetBuildInfo()) ~= WoWversion then
		MPS:UnregisterEvent("PVP_REWARDS_UPDATE")
		MPS:Echo("This addon is not compatible with this version of WoW. Please update it from curse.com")
	end
	
	
	
	if MPdb.char.MPSLoadProfileAtStart == 1 then
		MPS.testTimer = self:ScheduleRepeatingTimer("CheckMacroProfile", 4)
	end
	
end
	
-- ProfilesHub: Send message to all slaves that we are now exporting profile
function MPS:PreExport(name)
	MPS:SendAceMsg("export",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function MPS:Export(...)
	MPS:ExportCharProfile(...)
end

-- Export char profile to global
function MPS:ExportCharProfile(name)
	if MPS:ExistsProfile(name) then		
		local orig_profile = MPdb:GetCurrentProfile()
		
		MPglobaldb:SetProfile("General: "..name)
		MPdb:SetProfile(name)
		
		-- MPglobaldb.profile = MPdb.profile
		for index, value in orderedPairs(MPdb.profile) do
		  MPglobaldb.profile[index] = value
		end
	
		MPdb:SetProfile(orig_profile)
		
		return 1
	else
		MPS:Echo("No such profile exists: "..name)
		return nil
	end
end


-- TODO: Hook onto macrowindow to "lock" a macro
-- TODO: Hook onto macrowindow to select macros from a dropdown...ir if i should automate everything by spec.

function MPS:Echo(text)
	if PROFILES_TYST then return nil end
   DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Profiles: Macros - "..text)
   return 1
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreSaveMacroProfile(name)
	MPS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:SaveMacroProfile(name)
   
   -- Global DB Code
   local DB = {}
   if string.find(name,"General:") then 
      DB = MPglobaldb 
   else 
      DB = MPdb 
   end
   
   -- Save a restore point if empty db
   if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
      MPSSAVERESTORE=1 
      MPS:SaveMacroProfile("Restore [LOCKED]") 
      MPSSAVERESTORE=nil
   end
   
   -- User trying to overwrite restore point!
   if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
      MPS:Echo("This profile is locked at the first run and can only be loaded!")
      return nil
   end
   
   -- No macros!
   local numperglobal,numperchar = GetNumMacros()
   if numperchar == 0 and numperglobal == 0 then 
      MPS:Echo("Nothing to save!") 
      return nil 
   end
   
   -- Duplicate macro name!
   -- Since we are using the name to identify where its actionBar_pos is, WARN user
   
   DB:SetProfile(name)
   
   -- Solving /mp save tank X 2
   if MPS:ExistsProfile(name) then DB:ResetProfile(name) end
   
   MPS:Echo("Saving macros to profile "..name.."!")
   
   -- Ignore global macro: Ignore global macros
   local macroID_start = 1
   if MPdb.char.MPSIgnoreGlobalMacros == 1 then
      macroID_start = 121
   end
   
   -- Save macros
   for i=macroID_start,138 do
      
      -- No more macros saved
      if GetMacroInfo(i) then
         local macro_name, texture, body = GetMacroInfo(i)
         local actionBar_pos = {};
         
         -- Save actionbar position
         for actionID=1, 120 do
            local _, id = GetActionInfo(actionID)
            local abmacro_name, _, _ = GetMacroInfo(id)
            
            if i == id then
               table.insert(actionBar_pos , actionID)
            end
         end
         
         if not texture then texture = "INV_Misc_Note_01" end -- WOD: Seems like we have issues with textues
         texture = string.gsub(string.upper(texture), "INTERFACE\\ICONS\\", "");
         
         -- Save the macro to db
         DB.profile[i] = { macro_name,'60000',body,actionBar_pos,1,i,texture}
      end
   end
   
   MPS:SendMessage("MPS_Profiles_save",name) -- ProfilesHub
   return name
end


-- This is an alias for ProfilesHub
function MPS:Save(...)
	MPS:SaveMacroProfile(...)
end


-- ProfilesHub: Send message to all slaves that we are now loading profile
function MPS:PreLoadMacroProfile(name)
	MPS:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:LoadMacroProfile(name)
   
   
   -- Global DB Code
   local DB = {}
   if string.find(name,"General:") then 
      DB = MPglobaldb 
   else 
      DB = MPdb 
   end
   
   -- Save a restore point if empty db
   if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
      MPSSAVERESTORE=1 
      MPS:SaveMacroProfile("Restore [LOCKED]") 
      MPSSAVERESTORE=nil
   end
   
   -- No such profile saved
   if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return nil end
   
   -- In combat! 
   if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return nil end
   
   DB:SetProfile(name)
   MPdb.char.Active_profile = name
   
   MPS:Echo("Loading macros from profile "..name.."!")
   
   -- Ignore global macro: Get rid of all the current macros (unless we should ignore global)
   if MPdb.char.MPSIgnoreGlobalMacros == 0 then
      for i=1,120 do DeleteMacro(1)  end -- Global
   end
   
   for i=121,138 do DeleteMacro(121) end -- Char
   
   --Load saved macros profile
   -- Macro in DB - DB.profile[macroID] =
   --{
   --"Bomb!", -- [1] -- NAME
   --"60000", -- [2] -- VERSION
   --"#show Fel Armor\n/cast Demon Soul\n/cast Fel Armor\n/use Figurine - Jeweled Serpent\n", -- [3] -- BODY
   --{
   -- 47      --- ACTION BAR POSITION
   --}, -- [4]
   --1, -- [5]
   --1, -- [6] -- ID
   -- "SPELL_SHADOW_FELARMOUR", -- [7] -- ICON minus INTERFACE\ICONS\
   --}
   for index, keys in pairs(DB.profile) do   
      if DB.profile[index] then
         
         -- Patch 6.0
         if DB.profile[index][2] == '4.3' then 
            MPS:Compatibility_WOD(DB)
         end
         -- Patch <4.2 profiles, change all icons to questionmarks
         if not keys[7] then
            DB.profile[index][7] = 'INV_MISC_QUESTIONMARK'
            keys[7] = 'INV_MISC_QUESTIONMARK'
         end
         
         
         -- BUG: When we have these words in our macro, icon of the mod will not update!
         -- icon = 1
         if string.find(keys[3],"mod:")             then keys[7] = 'INV_MISC_QUESTIONMARK' end
         if string.find(keys[3],"modifier")         then keys[7] = 'INV_MISC_QUESTIONMARK' end
         if string.find(keys[3],"#showtooltip")     then keys[7] = 'INV_MISC_QUESTIONMARK' end
         
         local CreateMacroID = nil
         if index >= 121 then -- CreateMacro must know if its a global or char macro
            CreateMacroID     = CreateMacro(keys[1], keys[7], keys[3], 1);
         else
            -- Ignore global macro: Load global macro, unless options says NO
            if MPdb.char.MPSIgnoreGlobalMacros == 0 then
               CreateMacroID    = CreateMacro(keys[1], keys[7], keys[3], nil);
            end
            
         end
         
         
         if CreateMacroID == nil then
            CreateMacroID = index
         end
         
			-- Ignore global macro: Dont handle action bars if we ignore global macros
			if MPdb.char.MPSIgnoreGlobalMacros == 0 then
			 -- Action bar handling
			 if keys[4] then
				
				-- This macroid was marked as faulty at some point 
				if MPSMACROIDERROR and MPSMACROIDERROR == keys[6] then
				   MPS:DebugMsg("DB5 LoadMacroProfile() Moving Macro to Actionbar - MacroName: "..keys[1]) 
				   MPS:DebugMsg("DB6 LoadedMacroAttr="..inspect(DB.profile[index]))
				end
				
				for _,v in pairs(keys[4]) do
				   ClearCursor()
				   
				   -- Clear current spot
				   PickupAction(v)
				   ClearCursor()
				   
				   -- Restore this spot
				   PickupMacro(CreateMacroID)
				   PlaceAction(v)
				   ClearCursor()
				end
			end
         end
      end
   end
   
   
   if (IsAddOnLoaded("Blizzard_MacroUI")) then
      MacroFrame_Update() -- Blizzards own code
   end
   
   MPS:SendMessage("MPS_Profiles_load",name) -- ProfilesHub
   return name
end


-- Patch 6.0 | More global macros
function MPS:Compatibility_WOD(DB)
	-- Move macros from 37> to 121>
	-- Change version numbers on all loaded macros
	if WoWversion == 60000 then
		for index, keys in pairs(DB.profile) do  
			if DB.profile[index][2] == '4.3' then 
				DB.profile[index][2] = '60000' -- Change version numbers on all loaded macros
				 
				if index > 36 and index < 55 then 	-- Move macros from 37> to 121>
					new_index = index + 84
					DB.profile[new_index] = DB.profile[index]
					DB.profile[new_index][4] = DB.profile[index][4]
					DB.profile[new_index][6] = new_index
					DB.profile[index] = nil
				end	
			end
		end
	 end
end

-- This routine is loaded at startup to check if the profile was loaded correctly
-- It checks SavedVars versus Loaded macros
-- And forces a reload if it fails
-- 
-- What usually happens is that the MacroAPI lags behind leaving a non-consistent 
-- profile

function MPS:CheckMacroProfile(name, DebugLevel)
   MPS:DebugMsg("DB1 CheckMacroProfile() - Start") 
   name = name or MPdb.char.Active_profile
   DebugLevel = DebugLevel or 0
   local macro_integrity_check = true
   
   
   MPS:DebugMsg("DB1 CheckMacroProfile() - Active profile: "..name)
   MPS:DebugMsg("DB2 CheckMacroProfile() - Timer count: "..MPS.timerCount)
   
   -- Run this routine a set amount of times..
   MPS.timerCount = MPS.timerCount + 1
   if MPS.timerCount >= 4 then
		MPS:DebugMsg("CheckMacroProfile() Too many timer retries! CancelTimers!")
		MPS:CancelTimer(MPS.testTimer)
		return nil
   end
   
   -- Not preloading a RESTORE!
   if MPdb.char.Active_profile ~= "Restore [LOCKED]" then
      if MPSCheckMacroProfile_DEBUG == 0 then -- Dont reload a profile if faulty, when in debugmode
         MPS:LoadMacroProfile(name)
      end
      if MPSCheckMacroProfile_DEBUG == 1 then -- Max retries if in debugmode
         MPS.timerCount = 1
      end
   end
   
   -- Global DB Code
   local DB = {}
   if string.find(name,"General:") then 
      DB = MPglobaldb 
   else 
      DB = MPdb 
   end
   
   
   -- No such profile saved
   if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return nil end
   
   -- In combat! 
   if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return nil end
   
   DB:SetProfile(name)
   
   -- Check macro integrity against what is loaded
   for macroID, DBMacroAttr in pairs(DB.profile) do   -- Every macro in db 
      local LoadedMacroAttr = {GetMacroInfo(macroID)} -- Against all loaded macros
      
      
      -- Macro in DB - DBMacroAttr =
      --{
      --"Bomb!", -- [1] -- NAME
      --"60000", -- [2] -- VERSION
      --"#show Fel Armor\n/cast Demon Soul\n/cast Fel Armor\n/use Figurine - Jeweled Serpent\n", -- [3] -- BODY
      --{
      -- 47      --- ACTION BAR POSITION
      --}, -- [4]
      --1, -- [5]
      -- 1, -- [6] -- ID
      -- "SPELL_SHADOW_FELARMOUR", -- [7] -- ICON minus INTERFACE\ICONS\
      --}, -- [1]
      
      -- Loaded macro - LoadedMacroAttr =
      --{
      --"Bomb!", -- [1] -- NAME
      -- "INTERFACE\ICONS\SPELL_SHADOW_FELARMOUR", -- [2] -- ICON
      --"#show Fel Armor\n/cast Demon Soul\n/cast Fel Armor\n/use Figurine - Jeweled Serpent\n", -- [3] -- MACRO
      --}, 
      
      
      -- Cmp name
      if LoadedMacroAttr[1] ~= DBMacroAttr[1] then
         MPS:DebugMsg("DB5 CheckMacroProfile() Failed check LoadedMacroName vs SavedMacroName") 
         MPS:DebugMsg("DB6 LoadedMacroAttr="..inspect(LoadedMacroAttr))
         MPS:DebugMsg("DB6 DBMacroAttr = "..inspect(DBMacroAttr))   
         return nil 
      end 
      
      -- In rare cases (ie old profiles) the icon is loaded as nil! So we just gloss it over!
      if LoadedMacroAttr[2] == nil then 
         LoadedMacroAttr[2] = "INTERFACE\\ICONS\\"..DBMacroAttr[7]
         MPS:DebugMsg("DB6 CheckMacroProfile() LoadedIcon was nil! Replacing with what we saved in DB")
      end 
      
      -- Cmp icon
      if DBMacroAttr[7] ~= "INV_MISC_QUESTIONMARK" and string.upper(LoadedMacroAttr[2]) ~= "INTERFACE\\ICONS\\"..DBMacroAttr[7] then 
         print("Wrong name at macro icon "..index or "0") 
         MPS:DebugMsg("DB5 CheckMacroProfile() Failed check LoadedMacroIcon vs SavedMacroIcon") 
         MPS:DebugMsg("DB6 LoadedMacroAttr="..inspect(LoadedMacroAttr))
         MPS:DebugMsg("DB6 DBMacroAttr = "..inspect(DBMacroAttr))   
         return nil 
      end 
      
      -- Cmp body
      if LoadedMacroAttr[3] ~= DBMacroAttr[3] then 
         MPS:DebugMsg("DB5 CheckMacroProfile() Failed check LoadedMacroBody vs SavedMacroBody") 
         MPS:DebugMsg("DB6 LoadedMacroAttr="..inspect(LoadedMacroAttr))
         MPS:DebugMsg("DB6 DBMacroAttr = "..inspect(DBMacroAttr))   
         return nil 
      end 
      
      
      -- Actionbar checking
      for _,DBactionbarID in pairs(DBMacroAttr[4]) do
         local LoadedAB = {GetActionInfo(DBactionbarID)} -- = "macro", macroID
         
         -- Not even a macro *gasp*
         if LoadedAB[1] ~= "macro" then 
            MPS:DebugMsg("DB5 CheckMacroProfile() Failed check LoadedMacroActionBarPosTYPE vs DB") 
            MPS:DebugMsg("DB6 LoadedAB="..inspect(LoadedAB))
            MPS:DebugMsg("DB6 DBMacroAttr = "..inspect(DBMacroAttr)) 
            
            return nil
         end
         
         -- Check macro id
         if LoadedAB[2] ~= index then
			
            MPS:DebugMsg("DB5 CheckMacroProfile() Failed check LoadedMacroActionBarPosID vs DB") 
            MPS:DebugMsg("DB6 LoadedAB="..inspect(LoadedAB))
            MPS:DebugMsg("DB6 DBMacroAttr = "..inspect(DBMacroAttr))
			MPSMACROIDERROR = DBMacroAttr[6]
            return nil 
         end   
         
      end
      
   end 
   
   
   MPS:CancelTimer(MPS.testTimer)
   
   MPS:DebugMsg("Autoload timers are now off..")
   return true
end



-- This is an alias for ProfilesHub
function MPS:Load(...)
	MPS:LoadMacroProfile(...)
end

function MPS:ExistsProfile(name)
	-- Global DB Code
	local DB = {}
    if string.find(name,"General:") then
		DB = MPglobaldb 
	else 
		DB = MPdb 
	end
	
    for i, key in ipairs(DB:GetProfiles()) do
      if key == name then
         return (key)
      end
    end
   
    return nil
end

function MPS:ListProfiles(silent, globaldb)
    
    
	-- Global DB Code
	local DB = {}
    if globaldb then 	DB = MPglobaldb 
	else 				DB = MPdb 			end
	
	local MP_profiles = {}
	
	for i,k in ipairs(DB:GetProfiles()) do
		if not silent then 
			MPS:Echo(" Available Profiles: ") 
			MPS:Echo(' - ' .. k) 
		end
		
		MP_profiles[k] = 0
	end

	
	return MP_profiles
end

-- ProfilesHub: Send message to all slaves that we are now deleting a profile
function MPS:PreDeleteProfile(name)
	MPS:SendAceMsg("delete",name)
end

function MPS:DeleteProfil(name)

	-- Global DB Code
	local DB = {}
    if string.find(name,"General:") then 
		DB = MPglobaldb 
	else 
		DB = MPdb 
	end
	
	if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
		MPS:Echo("This profile cannot be deleted! It is locked at the first run and can only be loaded!")
		return nil
	end
	
	if (MPS:ExistsProfile(name)==nil)then
		MPS:Echo(name.." doesnt exist.")
		MPS:ListProfiles()
		
		return nil
	else
		if name == DB:GetCurrentProfile() then
			DB:SetProfile('Default')
		end
		DB:DeleteProfile(name)
		MPS:Echo(name.." deleted.")
		
		return name
	end
end

-- This is an alias for ProfilesHub
function MPS:Delete(...)
	MPS:DeleteProfil(...)
end


-- DEBUGGING FUNCTIONS
local MPSdebugframe = nil
local MPSdebugbuffer = {}
local MPSdebugEnabled = 0
function MPS:DebugWindowToggle()
   if MPdb.char.MPSdebugEnabled == 0 or MPdb.char.MPSdebugEnabled == nil then -- Toggle ENABLE
		-- if MPSdebugframe ~= 0 then MPSdebugframe:Release() end -- close already existing consoles
		MPSdebugEnabled = 1
		MPdb.char.MPSdebugEnabled=1


		MPSdebugframe = AceGUI:Create("Frame")
		MPSdebugframe:SetTitle("Profiles: Macros Debug")
		MPSdebugframe:SetStatusText("To turn this window off: /MPS debug off")
		MPSdebugframe:SetCallback("OnClose", function(widget) AceGUI:Release(widget) MPSdebugEnabled = 0 MPdb.char.MPSdebugEnabled=0 end)
		MPSdebugframe:SetLayout("Fill")
		MPSdebugframe:SetWidth("400")
		MPSdebugframe:SetHeight("400")
		MPS:DebugMsg("DebugWindowToggle(1)")
   else
		MPS:DebugMsg("DebugWindowToggle(0)")
		MPdb.char.MPSdebugEnabled=0
		MPSdebugframe:Release()
   end
   
end

function MPS:DebugMsg(...)

	debugMessage = ...
	-- DebugLevel - A debugMessage prefixed with DB%d will set the DebugLevel
	-- From a scale of 0 (None) to 5 (blurt everything) and 6 (Dev mumbojambo)
	local DebugLevel,macroID 	= tonumber(string.match(debugMessage,"DB(%d) ")) or 0
	debugMessage 				= tostring(string.gsub(debugMessage,"DB(%d) ","")) or "0"
	
	if DebugLevel >= 5 and MPSDEV == 6 then
		print(debugMessage.." - DLEVEL"..DebugLevel)
	end
				
	if MPSdebugbuffer == 0 then
	  debugMessage = date("%H:%M:%S").." - DEBUGGER Loaded"
	  table.insert(MPSdebugbuffer,debugMessage)
	  
	else
	  debugMessage = date("%H:%M:%S").." - "..debugMessage
	  table.insert(MPSdebugbuffer,debugMessage)
	end

	if MPSdebugEnabled ~= 1 then 
	  MPSdebugbuffer = MPS:circular_list(MPSdebugbuffer,200) -- Save 20 debug messages by default, even with debug turned off.
	  return
	end

	MPSdebugframe:ReleaseChildren() -- Refresh textbox
	-- local debugDropdown = AceGUI:Create("Dropdown")
	-- debugDropdown:SetText("Debug level")
	-- debugDropdown:SetWidth(20)
	-- debugDropdown:SetList({
	  -- ["choice1"] = "Text for Choice #1",
	  -- ["choice2"] = "Text for Choice #2",
	  -- and so on
	-- })
	-- debugDropdown:SetValue("choice1") -- choice1 is checked initialy

	-- debugDropdown:SetCallback("OnValueChanged", function(choice)
	  -- choice is the list index of the selected item
	-- end )
	-- MPSdebugframe:AddChild(debugDropdown)
	
	-- local editbox = AceGUI:Create("MultiLineEditBox")
	-- editbox:SetNumLines(10)
	-- local MPSdebugbuffervariable = "\n"
	-- for _,value in ipairs(MPSdebugbuffer) do 
	  -- MPSdebugbuffervariable =  MPSdebugbuffervariable..value.."\n" 
	-- end
	-- editbox:SetText(MPSdebugbuffervariable)
	-- editbox:DisableButton(1)
	-- MPSdebugframe:AddChild(editbox)

	

end

-- Give this function a list_objec = {} and max amount of entries in that table
-- and it will remove them for you
function MPS:circular_list(list_object,list_max)
   
   if (#list_object > list_max)then
      while #list_object > list_max do 
         table.remove(list_object,1)
      end
   end
   
   return list_object
   
end